# This file was auto-generated by Fern from our API Definition.

import typing

from .actions_create_request_selected_items_excluded import ActionsCreateRequestSelectedItemsExcluded
from .actions_create_request_selected_items_included import ActionsCreateRequestSelectedItemsIncluded

ActionsCreateRequestSelectedItems = typing.Union[
    ActionsCreateRequestSelectedItemsIncluded, ActionsCreateRequestSelectedItemsExcluded
]
