# This file was auto-generated by Fern from our API Definition.

import typing

AnnotationLastAction = typing.Union[
    typing.Literal[
        "prediction",
        "propagated_annotation",
        "imported",
        "submitted",
        "updated",
        "skipped",
        "accepted",
        "rejected",
        "fixed_and_accepted",
        "deleted_review",
    ],
    typing.Any,
]
