# This file was auto-generated by Fern from our API Definition.

import typing

ViewsCreateRequestDataFiltersItemsItemFilter = typing.Union[
    typing.Literal[
        "filter:tasks:agreement",
        "filter:tasks:annotations_results",
        "filter:tasks:annotators",
        "filter:tasks:cancelled_annotations",
        "filter:tasks:comments",
        "filter:tasks:completed_at",
        "filter:tasks:created_at",
        "filter:tasks:file_upload",
        "filter:tasks:ground_truth",
        "filter:tasks:id",
        "filter:tasks:inner_id",
        "filter:tasks:predictions_model_versions",
        "filter:tasks:predictions_results",
        "filter:tasks:predictions_score",
        "filter:tasks:reviewed",
        "filter:tasks:reviewers",
        "filter:tasks:reviews_accepted",
        "filter:tasks:reviews_rejected",
        "filter:tasks:total_annotations",
        "filter:tasks:total_predictions",
        "filter:tasks:unresolved_comment_count",
        "filter:tasks:updated_at",
    ],
    typing.Any,
]
