# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class TaskFilterOptions(pydantic_v1.BaseModel):
    view: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Apply filters from the view ID (a tab from the Data Manager)
    """

    skipped: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    `only` - include all tasks with skipped annotations<br>`exclude` - exclude all tasks with skipped annotations
    """

    finished: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    `only` - include all finished tasks (is_labeled = true)<br>`exclude` - exclude all finished tasks
    """

    annotated: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    `only` - include all tasks with at least one not skipped annotation<br>`exclude` - exclude all tasks with at least one not skipped annotation
    """

    only_with_annotations: typing.Optional[bool] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
