# This file was auto-generated by Fern from our API Definition.

from .azure_create_response import AzureCreateResponse
from .azure_update_response import AzureUpdateResponse

__all__ = ["AzureCreateResponse", "AzureUpdateResponse"]
