# This file was auto-generated by Fern from our API Definition.

import typing

WebhooksUpdateRequestActionsItem = typing.Union[
    typing.Literal[
        "PROJECT_CREATED",
        "PROJECT_UPDATED",
        "PROJECT_DELETED",
        "TASKS_CREATED",
        "TASKS_DELETED",
        "ANNOTATION_CREATED",
        "ANNOTATIONS_CREATED",
        "ANNOTATION_UPDATED",
        "ANNOTATIONS_DELETED",
        "LABEL_LINK_CREATED",
        "LABEL_LINK_UPDATED",
        "LABEL_LINK_DELETED",
    ],
    typing.Any,
]
