# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .webhook_actions_item import WebhookActionsItem


class Webhook(pydantic_v1.BaseModel):
    id: typing.Optional[int] = None
    organization: typing.Optional[int] = None
    project: typing.Optional[int] = None
    url: str = pydantic_v1.Field()
    """
    URL of webhook
    """

    send_payload: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    If value is False send only action
    """

    send_for_all_actions: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    If value is False - used only for actions from WebhookAction
    """

    headers: typing.Optional[typing.Dict[str, typing.Any]] = pydantic_v1.Field(default=None)
    """
    Key Value Json of headers
    """

    is_active: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    If value is False the webhook is disabled
    """

    actions: typing.Optional[typing.List[WebhookActionsItem]] = None
    created_at: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Creation time
    """

    updated_at: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Last update time
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
