# This file was auto-generated by Fern from our API Definition.

import typing

ActionsCreateRequestId = typing.Union[
    typing.Literal[
        "retrieve_tasks_predictions",
        "predictions_to_annotations",
        "remove_duplicates",
        "delete_tasks",
        "delete_ground_truths",
        "delete_tasks_annotations",
        "delete_tasks_reviews",
        "delete_tasks_predictions",
        "delete_reviewers",
        "delete_annotators",
    ],
    typing.Any,
]
