# This file was auto-generated by Fern from our API Definition.

import typing

ViewsCreateRequestDataOrderingItem = typing.Union[
    typing.Literal[
        "tasks:agreement",
        "tasks:annotations_results",
        "tasks:annotators",
        "tasks:cancelled_annotations",
        "tasks:comments",
        "tasks:completed_at",
        "tasks:created_at",
        "tasks:file_upload",
        "tasks:ground_truth",
        "tasks:id",
        "tasks:inner_id",
        "tasks:predictions_model_versions",
        "tasks:predictions_results",
        "tasks:predictions_score",
        "tasks:reviewed",
        "tasks:reviewers",
        "tasks:reviews_accepted",
        "tasks:reviews_rejected",
        "tasks:total_annotations",
        "tasks:total_predictions",
        "tasks:unresolved_comment_count",
        "tasks:updated_at",
    ],
    typing.Any,
]
