# This file was auto-generated by Fern from our API Definition.

import typing

AzureBlobImportStorageStatus = typing.Union[
    typing.Literal["initialized", "queued", "in_progress", "failed", "completed"], typing.Any
]
