# This file was auto-generated by Fern from our API Definition.

import typing

ViewsUpdateRequestDataFiltersItemsItemOperator = typing.Union[
    typing.Literal[
        "contains",
        "ends_with",
        "equal",
        "exists",
        "greater",
        "greater_or_equal",
        "in",
        "less",
        "less_or_equal",
        "not_contains",
        "not_equal",
        "not_exists",
        "not_in",
        "starts_with",
    ],
    typing.Any,
]
