# This file was auto-generated by Fern from our API Definition.

import typing

ProjectSampling = typing.Union[
    typing.Literal["Sequential sampling", "Uniform sampling", "Uncertainty sampling"], typing.Any
]
